package com.xxx.workflow;

import org.jbpm.graph.def.ActionHandler;
import org.jbpm.graph.exe.ExecutionContext;
import org.jbpm.graph.exe.ProcessInstance;

/**
 * @author chadwickboggs@yahoo.com
 */
public class LaunchProcessActionHandler extends ActionHandlerSupport implements ActionHandler {

	private String definition;
    private String definitionVar;
    private String propogate;
    private boolean signal;
    private String signalVar;
    private boolean leave;
    private String leavelVar;

    public String getDefinition() {
        return definition;
    }

    public void setDefinition(final String definition) {
        this.definition = definition;
    }

    public String getDefinitionVar() {
        return definitionVar;
    }

    public void setDefinitionVar(final String definitionVar) {
        this.definitionVar = definitionVar;
    }

    public String getPropogate() {
        return propogate;
    }

    public void setPropogate(final String propogate) {
        this.propogate = propogate;
    }

    public boolean isSignal() {
        return signal;
    }

    public void setSignal(boolean signal) {
        this.signal = signal;
    }

    public String getSignalVar() {
        return signalVar;
    }

    public void setSignalVar(String signalVar) {
        this.signalVar = signalVar;
    }

    public boolean isLeave() {
        return leave;
    }

    public void setLeave(boolean leave) {
        this.leave = leave;
    }

    public String getLeavelVar() {
        return leavelVar;
    }

    public void setLeavelVar(String leavelVar) {
        this.leavelVar = leavelVar;
    }

    public void execute(final ExecutionContext executionContext) throws Exception {
        initializeDefinition(executionContext);

        final JbpmSession session = executionContext.getProcessInstance().getJbpmSession();
        final ProcessDefinition processDefinition = session.getGraphSession().findLatestProcessDefinition(definition);
        ProcessInstance processInstance = processDefinition.createProcessInstance();
        session.getGraphSession().saveProcessInstance(processInstance);
        propogateVariables(processInstance, executionContext);
        if (signal) {
            processInstance.signal();
        }
        if (leave) {
            executionContext.getNode().leave(executionContext);
        }
    }

    private void initializeDefinition(final ExecutionContext executionContext) {
        initialize("definition", definitionVar, executionContext);
        initialize("signal", signalVar, executionContext);
        initialize("leave", leavelVar, executionContext);
    }

    private void propogateVariables(final ProcessInstance processInstance, final ExecutionContext executionContext)
            throws Exception {
        if (propogate == null || propogate.trim().length() == 0) {
            return;
        }

        final String[] names = propogate.split(",");
        for (int i = 0; i < names.length; i++) {
            final Object value = executionContext.getContextInstance().getVariable(names[i], executionContext.getToken());
            if (value == null) {
                throw new IllegalArgumentException("Process variable '" + names[i] + "' can not be propogated " +
                        "to from process '" + executionContext.getProcessDefinition().getName() + "' to process '" +
                        processInstance.getProcessDefinition().getName() + "' because it was not found.");
            }

            processInstance.getContextInstance().createVariable(names[i], value);
        }
    }

}
